@@ -28,7 +28,7 @@ rawSNP <- m1$data$observed
 pgen <- read.table(file.path(tdir, "out.log"), as.is = TRUE, header=TRUE,
                      sep="\t", check.names=FALSE, quote="", comment.char="")
 expect_equal(nrow(pgen), 199)
-expect_equal(m1$compute$steps$LD$debug$loadCounter, 1)
+expect_equal(m1$compute$steps[[2]]$debug$loadCounter, 1)
 expect_equal(range(m1$data$observed$snp), c(0,2), .01)
 
 m1 <- GWAS(buildOneFac(pheno, paste0("i", 1:numIndicators)),
@@ -44,7 +44,7 @@ test_that("bed SNP data", {
 bed <- read.table(file.path(tdir, "out.log"), as.is = TRUE, header=TRUE,
                    sep="\t", check.names=FALSE, quote="", comment.char="")
 expect_equal(nrow(bed), 199)
-expect_equal(m1$compute$steps$LD$debug$loadCounter, 1)
+expect_equal(m1$compute$steps[[2]]$debug$loadCounter, 1)
 lr <- loadResults(file.path(tdir, "out.log"), focus = "snp_to_F", signAdj="lambda_i1")
 expect_equal(colnames(lr), c("MxComputeLoop1", "CHR", "BP", "SNP", "A1", "A2",
                              "statusCode", "catch1", "snp_to_F", "Z", "P"))
@@ -57,7 +57,7 @@ expect_equal(rawSNP$snp, m1$data$observed$snp, tolerance=5e-5)
 bgen <- read.table(file.path(tdir, "out.log"), as.is = TRUE, header=TRUE,
                   sep="\t", check.names=FALSE, quote="", comment.char="")
 expect_equal(nrow(bgen), 199)
-expect_equal(m1$compute$steps$LD$debug$loadCounter, 1)
+expect_equal(m1$compute$steps[[2]]$debug$loadCounter, 1)
 lr <- loadResults(file.path(tdir, "out.log"), "snp_to_F", signAdj='lambda_i1')
 expect_equal(colnames(lr), c("MxComputeLoop1", "CHR", "BP", "SNP", "A1", "A2",
                              "statusCode", "catch1", "snp_to_F", "Z", "P"))
@@ -71,7 +71,7 @@ m2 <- GWAS(buildOneFac(pheno, paste0("i", 1:numIndicators)),
 last <- read.table(file.path(tdir, "out.log"), as.is = TRUE, header=TRUE,
                    sep="\t", check.names=FALSE, quote="", comment.char="")
 expect_equal(nrow(last), 10)
-expect_equal(m2$compute$steps$LD$debug$loadCounter, 1)
+expect_equal(m2$compute$steps[[2]]$debug$loadCounter, 1)
 expect_equal(last[['example.bgen:SNP']],
              bgen[['example.bgen:SNP']][190:199])
 lr <- loadResults(file.path(tdir, "out.log"), "snp_to_F", signAdj='lambda_i1')
